ui = {}
ui.textureTree = nil
if love.filesystem.getInfo("fonts/font.ttf") then
	ui.smallFont = love.graphics.newFont("fonts/font.ttf", 14)
	ui.font = love.graphics.newFont("fonts/font.ttf", 20)
	ui.bigFont = love.graphics.newFont("fonts/font.ttf", 45)
else
	ui.smallFont = love.graphics.getFont(14)
	ui.font = love.graphics.getFont(20)
	ui.bigFont = love.graphics.getFont(45)
end


function ui.update_scale(p_scale)
    scale = p_scale
end


ui.helptext = [[[P] toggle fly
[A/D] Move
[Hold Shift] Move faster
[Space] Jump
[A/S/DW] Move (flying)
[C] Clear canvas
[F] Fill foreground
[G] Fill background
[Mouse1] Draw
[Mouse2] Delete
[Hold ctrl] Draw on Background
[+/-/PGUP/PGDWN] Zoom canvas
[Cursor keys] Move camera
]]


local textEnabled = false
local textEntry = ""
function ui.keyreleased(key, _)
	if key == "return" and not love.keyboard.isDown("lalt") then
		textEnabled = true
		love.keyboard.setTextInput(true)
		ui.draw(window.x, window.y)
	end
	if textEnabled then
		love.visible = function(visible)
			if visible then
				love.update = function(dt)
					gameloop.networkSync(dt)
					drawing.input(dt)
					physics.update(dt)
					camera.update(dt)
				end
			else
				love.update = function(dt)
					gameloop.networkSync(dt)
					physics.update(dt)
				end
			end
		end
		love.visible(true)
		love.keyreleased = function(key, _)
			if key == "return" and not love.keyboard.isDown("lalt") then
				textEnabled = false
				if textEntry ~= "" then
					gameloop.networkSend(unit("chatMessage", textEntry))
					textEntry = ""
				end
				ui.draw(window.x, window.y)
				love.keyboard.setTextInput(false)
				love.keyreleased = gameloop.keyreleased
				love.keypressed = gameloop.keypressed
				love.update = gameloop.normalVisible
			elseif key == "backspace" then
				textEntry = string.sub(textEntry, 0, #textEntry -1)
				ui.draw(window.x, window.y)
			end
		end
		love.keypressed = function(key, _) return end
	else
		love.keyreleased = gameloop.keyreleased
		love.keypressed = gameloop.keypressed
	end
end


function love.textinput(text)
    textEntry = textEntry .. text
	ui.draw(window.x, window.y)
end


function ui.chatEntry()
	local passedEntry = ""
	if textEntry == "" then passedEntry = "Start typing your message." else passedEntry = textEntry end
	local chatEntry =
		{name = "copySize",
			{name = "rect",
				fill = "fill",
				color = { r = .1, g = .1, b = .1, .7 }
			},
		{name = "frame",
			{name = "label",
				text = passedEntry
			}
		}
	}
	return chatEntry
end


chatentries = {name = "vertical", {name = "spacer", width = 0}}
function ui.chatlog()
	return chatentries
end


function ui.addChatEntry(user, message)
	if #chatentries > 10 then
		table.remove(chatentries, 1)
	end
	local chatText
	if user ~= "" then
		chatText = user .. ": " .. message
	else
		chatText = message
	end
	local entry = {name = "rotate",
		rotation = 0, -- OKAY DIEGO >:(
			{name = "copySize",
				{name = "rect",
					fill = "fill",
					color = { r = .1, g = .1, b = .1, a = .6 }
				},
				{name = "vertical",
					{name = "label",
						text = chatText,
						font = ui.smallFont
					},
					{name = "spacer",
						height = 2
					}
				}
			}
		}
	table.insert(chatentries, entry)
	ui.draw(window.x, window.y)
end


function ui.loadTextures(directory, container)
	if not container then container = {} end
	local entries = love.filesystem.getDirectoryItems(directory)
	if #entries == 0 then return container end
	for i, file in pairs(entries) do
		local path = directory .."/".. file
		local entry = love.filesystem.getInfo(path)
		if entry.type == "directory" then
			container[file] = {}
			container[file] = ui.loadTextures(path, container[file])
		elseif entry.type == "file" or entry.type == "symlink" then
			if file ~= "license.txt" then
				local imageData = love.image.newImageData(path)
				local hash = love.data.hash("sha512", imageData:getString())
				gameloop.textures[hash] = {data = imageData}
				container[file] = hash
			end
		elseif entry.type == "other" then
			error("Can't figure out what " .. path .. " is!")
		end
	end
	return container
end


function textureEntry(hash)
	local texture = gameloop.textures[hash]
	assert(texture, "No texture found for textureentry?")
	assert(texture.data, "No imagedata found for textureentry?")
	if not texture.image then
		texture.image = love.graphics.newImage(texture.data)
	end
	local width = texture.image:getWidth()
	local scale = 1
	if width <= 16 then scale = 2 end
	if width <= 8 then scale = 4 end
	if drawing and drawing.cursor and hash == drawing.cursor then
		return{name = "overlayRect",
			fill = "line",
			{name = "drawHash",
				hash = hash,
				scale = scale
			}
		}
	else
		return{name = "cursorSelect",
			identifier = hash,
			kind = "picker",
			{name = "drawHash",
				hash = hash,
				scale = scale
			}
		}
	end
end


function textureRun(entryTable, width)
	local textures = gameloop.textures
	local superEntry = {name = "vertical"}
	local entries = {name = "horizontal"}
	local entryTableSorted = {}
	for iter, entry in pairs(entryTable) do
		table.insert(entryTableSorted, {name = iter, entry = entry})
	end
	table.sort(entryTableSorted, function(k1, k2) return k1.name < k2.name end)
	for iter, entry in pairs(entryTableSorted) do
		if #entries > 2 then
			local NextX, nextY = layout.handle(entries, 0, 0, true)
			if not textures[entry.entry] then textures[entry.entry] = {} end
			if not textures[entry.entry].image then
				textures[entry.entry].image = love.graphics.newImage(textures[entry.entry].data)
			end
			if textures[entry.entry].image:getWidth() + NextX < width then
				table.insert(entries, textureEntry(entry.entry))
			else
				table.insert(superEntry, entries)
				entries = {name = "horizontal"}
			end
		else
			table.insert(entries, textureEntry(entry.entry))
		end
	end
	if #entries > 1 then table.insert(superEntry, entries) end
	if #superEntry > 1 then
		return superEntry
	else
		return entries
	end
end


function ui.blockDrawer(textureTree, w, h, container)
	if not textureTree then return {name = "label", text = "Loading textures..."} end
	if not container then container = {name="vertical",
		{name = "colorPicker", kind = "colorpicker", granularity = 0.25}}
	end
	for author, tree in pairs(textureTree) do
		for pack, subTree in pairs(tree) do
			container.name = "vertical"
			table.insert(container, {name = "label", text = pack})
			table.insert(container,
				{name = "color",
					color = drawing.color,
					textureRun(subTree, w)
				})
		end
	end
	return container
end


function ui.helpDrawer(w, h)
	return {name = "label",
		text = ui.helptext
	}
end


function ui.testDrawer(w, h)
	local drawer = {name = "vertical"}
	return drawer
end


local selectedTab = "help"
function ui.tabDrawer(textureTree, w, h)
	local labels
	local helpTab = {name = "label",
		text = ui.helptext
		}
	local hWidth, hHeight = layout.handle(helpTab, 0, 0, true)
	local blockTab = ui.blockDrawer(textureTree, w, h)
	local bWidth, bHeight = layout.handle(blockTab, 0, 0, true)
		labels = {name = "horizontal",
			{name = "cursorSelect",
				identifier = "help",
				kind = "tab",
				{name = "label",
					text = " [Help] "
				}
			 },
			 {name = "cursorSelect",
				identifier = "blocks",
				kind = "tab",
				{name = "label",
					text = " [Blocks] "
				}
			},
			{name = "cursorSelect",
				identifier = "disconnect",
				kind = "button",
				{name = "label",
					text = " Disconnect. "
				}
			}
		}
	if selectedTab == "blocks" then
		tab = blockTab
	elseif selectedTab == "help" then
		tab = helpTab
	end
	local drawer = {
		name = "vertical",
		{name = "copySize",
			{name = "rect",
				fill = "fill",
				color = { r = .2, g = 0, b = .2 }
			},
			{name = "horizontal",
				labels,
				{name = "spacer",
					height = 40,
				},
			},
		},
		{name = "copySize",
			{name = "rect",
				fill = "fill",
				color = { r = .1, g = .1, b = .1 }
			},
			tab
		}
	}
	return drawer, math.max(hWidth, bWidth)
end


function ui.draw(w, h)
	window.x, window.y = w, h
	color_push()
	love.graphics.setCanvas(ui.buffer)
		uiState = {}
		love.graphics.clear( )
		local sidebar, width = ui.tabDrawer(ui.textureTree, w, h)
		layout.handle(sidebar, window.x -width, 0)
		ui.space = width
		if textEnabled then
			local chatLayout = ui.chatEntry()
			local sizex, sizey = layout.handle(chatLayout, 0, 0, true)
			layout.handle(chatLayout, 0, window.y - sizey)
		end
		layout.handle(ui.chatlog(), 25, 90)
	color_pop()
	love.graphics.setCanvas()
end


function ui.mousepressed(mousex, mousey)
	local textures = gameloop.textures
	for i, v in ipairs(uiState) do
		if mousex >= v.x and mousex <= v.x2 then
			if mousey >= v.y and mousey <= v.y2 then
				if v.kind == "picker" then
					assert(v.identifier, "No identifier in picker!")
					drawing.cursorHash = v.identifier
					drawing.cursor = textures[v.identifier]
					assert(drawing.cursor, "No cursor texture found!")
					ui.draw(window.x, window.y)
					return
				elseif v.kind == "tab" then
					selectedTab = v.identifier
					ui.draw(window.x, window.y)
					return
				elseif v.kind == "colorpicker" then
					local scale = 1.0 / granularity
					local red = math.floor((mousey - v.y) / v.pointsize) / scale
					local green = math.floor((mousex - v.x) / v.pointsize / (scale + 1)) / scale
					local blue = (math.floor((mousex - v.x) / v.pointsize) % (scale + 1)) / scale
					drawing.color = {red, green, blue, 1}
					ui.draw(window.x, window.y)
					return
				elseif v.kind == "button" then
					if v.identifier == "disconnect" then
						gameloop.networkSend(unit("playerLeave"))
						menu.init()
					end
				end
			end
		end
	end
end


ui.init = function()
	love.keyboard.setTextInput( false )
	local w,h = love.window.getMode()
	ui.textureTree = ui.loadTextures("textures/blocks")

	ui.draw(w, h)
	ui.resize = function()
		local w,h = love.window.getMode()
		ui.buffer = love.graphics.newCanvas(w, h)
		love.graphics.setCanvas(ui.buffer)
			love.graphics.clear()
			ui.draw(w, h)
		love.graphics.setCanvas()
	end
end

local w, h = love.window.getMode()
ui.buffer = love.graphics.newCanvas(w, h)
ui.space = 300
